import logging
import tempfile
import zipfile
from configparser import ConfigParser

import os
from distutils.dir_util import copy_tree
from xml.etree import ElementTree

import requests
import json

import shutil
from requests.auth import HTTPBasicAuth


class GeoServer:
    def __init__(self, configFile, debug,  logName='GeoServer.log'):
        self.layerData = None
        self.workspaceData = None
        self.coveragestoreData = None
        self.initLogger(debug, logName)
        self.parseConfig(configFile)

    def initLogger(self, debug, logName):

        self.logger = logging.getLogger('GeoServer')

        fh = logging.FileHandler(logName)

        if debug:
            self.logger.setLevel(logging.DEBUG)
            fh.setLevel(logging.DEBUG)
        else:
            self.logger.setLevel(logging.INFO)
            fh.setLevel(logging.INFO)

        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        fh.setFormatter(formatter)
        self.logger.addHandler(fh)

    def parseConfig(self, configFile):

        self.cfg = ConfigParser()
        self.logger.info('Reading configuration file at %s' % configFile)
        self.cfg.read(configFile)

    def getLayers(self):
        apiUrl = '{0}/layers'.format(self.cfg.get('GeoServer', 'url'))

        self.logger.info('Getting layers from %s' % apiUrl)
        response = requests.get(apiUrl, auth=HTTPBasicAuth(self.cfg.get('GeoServer', 'username'),
                                                           self.cfg.get('GeoServer', 'password')))
        if response.status_code == 200:
            jsonData = json.loads(response.content.decode('utf-8'))
            self.logger.debug(jsonData)
            return jsonData
        else:
            self.logger.error('Could not retrieve layers - Status Code = %d' % response.status_code)
            return None

    def getWorkspaces(self):
        apiUrl = '{0}/workspaces'.format(self.cfg.get('GeoServer', 'url'))

        self.logger.info('Getting workspaces from %s' % apiUrl)
        response = requests.get(apiUrl, auth=HTTPBasicAuth(self.cfg.get('GeoServer', 'username'),
                                                           self.cfg.get('GeoServer', 'password')))
        if response.status_code == 200:
            jsonData = json.loads(response.content.decode('utf-8'))
            self.logger.debug(jsonData)
            return jsonData
        else:
            self.logger.error('Could not retrieve workspaces - Status Code = %d' % response.status_code)
            return None

    def getCoverageStores(self, workspaceName):
        apiUrl = '{0}/workspaces/{1}/coveragestores'.format(self.cfg.get('GeoServer', 'url'), workspaceName)

        requestHeaders = {'Content-Type': 'application/json'}
        self.logger.info('Getting coverage stores from %s' % apiUrl)
        response = requests.get(apiUrl, headers=requestHeaders, auth=HTTPBasicAuth(self.cfg.get('GeoServer', 'username'),
                                                           self.cfg.get('GeoServer', 'password')))
        if response.status_code == 200:
            jsonData = json.loads(response.content.decode('utf-8'))
            self.logger.debug(jsonData)
            return jsonData
        else:
            self.logger.error('Could not retrieve workspaces - Status Code = %d' % response.status_code)
            return None

    def layerExists(self, layerName):
        if not self.layerData:
            self.layerData = self.getLayers()

        return (len(self.layerData["layers"]) and len(
            [layer for layer in self.layerData["layers"]["layer"] if layer["name"] == layerName]) > 0)

    def workspaceExists(self, workspaceName):
        if not self.workspaceData:
            self.workspaceData = self.getWorkspaces()

        return (len(self.workspaceData["workspaces"]) and len(
            [ws for ws in self.workspaceData["workspaces"]["workspace"] if ws["name"] == workspaceName]) > 0)

    def coverageStoreExists(self, workspaceName, csName):
        if not self.coveragestoreData:
            self.coveragestoreData = self.getCoverageStores(workspaceName)

        return (len(self.coveragestoreData["coverageStores"]) and len(
            [cs for cs in self.coveragestoreData["coverageStores"]["coverageStore"] if cs["name"] == csName]) > 0)

    def createWorkspace(self, workspaceName):
        apiUrl = '{0}/workspaces/'.format(self.cfg.get('GeoServer', 'url'))

        requestHeaders = {'Content-Type': 'application/json'}
        requestData = json.dumps({"workspace": {"name": workspaceName}})

        self.logger.info('Creating workspace at %s' % apiUrl)
        response = requests.post(apiUrl, headers=requestHeaders,
                                 auth=HTTPBasicAuth(self.cfg.get('GeoServer', 'username'),
                                                    self.cfg.get('GeoServer', 'password')), data=requestData)
        if response.status_code == 201:
            responseData = response.content.decode()
            self.logger.debug(responseData)
            self.logger.info('Created %s workspace' % responseData)
            self.logger.info('Reloading workspaces')
            self.workspaceData = self.getWorkspaces()
        else:
            self.logger.error('Could not create workspace %s - Status Code = %d' % (workspaceName,response.status_code))

    def createCoverageStore(self, workspaceName, csName, imageUrl):
        apiUrl = '{0}/workspaces/{1}/coveragestores/{2}/file.imagemosaic'.format(self.cfg.get('GeoServer', 'url'), workspaceName, csName)

        requestHeaders = {'Content-Type': 'application/zip'}
        requestData = open(self.createCoverageStoreZip(csName, imageUrl), 'rb').read()
        self.logger.info('Creating coverage store at %s' % apiUrl)
        response = requests.put(apiUrl, headers=requestHeaders,
                                 auth=HTTPBasicAuth(self.cfg.get('GeoServer', 'username'),
                                                    self.cfg.get('GeoServer', 'password')), data=requestData)
        if response.status_code == 201:
            responseData = response.content.decode()
            self.logger.debug(responseData)
            self.logger.info('Created %s coveragestore' % responseData)
            self.logger.info('Reloading coveragestore')
  #          self.coveragestoreData = self.getCoverageStores(workspaceName)
        else:
            self.logger.error(
                'Could not create coveragestore %s/%s - Status Code = %d' % (workspaceName, csName, response.status_code))

    def createCoverageStoreZip(self, csName, imageUrl):
        tmpDir = tempfile.mkdtemp()
        shutil.copy(imageUrl, tmpDir)
        copy_tree(os.path.abspath(self.cfg.get('CoverageStore', 'properties') + csName), tmpDir)

        tmpDirZip = tempfile.mkdtemp()
        tmpZipPath = tmpDirZip + "/properties.zip"
        tmpZip = zipfile.ZipFile(tmpZipPath, "w")

        for dirname, _, files in os.walk(tmpDir):
            for file in files:
                tmpZip.write(dirname + "/" + file, file)

        tmpZip.close()
        return tmpZipPath

    def registerTile(self, workspaceName, layerName, imageUrl):

        apiUrl = '{0}/workspaces/{1}/coveragestores/{2}/external.imagemosaic'.format(self.cfg.get('GeoServer', 'url'),
                                                                                 workspaceName, layerName)

        requestHeaders = {'Content-Type': 'text/plain'}
        requestData = 'file://%s' % os.path.abspath(imageUrl)
        self.logger.debug(requestData)
        self.logger.info('Registering tile at %s' % apiUrl)
        response = requests.post(apiUrl, headers=requestHeaders,
                                auth=HTTPBasicAuth(self.cfg.get('GeoServer', 'username'),
                                                   self.cfg.get('GeoServer', 'password')), data=requestData)
        if response.status_code == 200 or response.status_code == 202 :
            responseData = response.content.decode()
            self.logger.debug(responseData)
            self.logger.info('Registered %s tile' % responseData)
        else:
            self.logger.error(
                'Could not register tile to %s/%s - Status Code = %d' % (workspaceName, layerName, response.status_code))

    def getRegisteredTiles(self, workspaceName, layerName):

        apiUrl = '{0}/workspaces/{1}/coveragestores/{2}/coverages/{2}/index/granules.xml?limit=200'.format(self.cfg.get('GeoServer', 'url'), workspaceName, layerName)

        self.logger.info('Getting registered tiles from %s' % apiUrl)
        response = requests.get(apiUrl, auth=HTTPBasicAuth(self.cfg.get('GeoServer', 'username'),
                                                           self.cfg.get('GeoServer', 'password')))
        if response.status_code == 200:
            xmlData = ElementTree.ElementTree(ElementTree.fromstring(response.text.replace('\r\n', '')))
            self.logger.debug(xmlData)
            return xmlData
        else:
            self.logger.error('Could not retrieve registered tiles - Status Code = %d' % response.status_code)
            return None
