#!/usr/bin/env python

import pandas as pd
import numpy as np
import rasterio
import os
import logging
import sys
import re
from configparser import ConfigParser
from pathlib import Path
from rasterio.transform import from_origin, from_bounds
from datetime import datetime


####################### CLASSES #############################################################


class Rasterizer:
    def __init__(self, configFile, debug, logName='Rasterizer.log'):
        self.initLogger(debug, logName)
        self.parseConfig(configFile)

    def initLogger(self, debug, logName):
        self.logger = logging.getLogger('Rasterizer')
        fh = logging.FileHandler(logName)
        if debug:
            self.logger.setLevel(logging.DEBUG)
            fh.setLevel(logging.DEBUG)
        else:
            self.logger.setLevel(logging.INFO)
            fh.setLevel(logging.INFO)

        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        fh.setFormatter(formatter)

        self.logger.addHandler(fh)

    def parseConfig(self, configFile):
        self.cfg = ConfigParser()
        self.logger.info('Reading configuration file at %s' % configFile)
        self.cfg.read(configFile)

    def rasterizeECMWFMeteoCSV(self, inputFile, outputDir):

        try:
            self.logger.info('Rasterizing datafile %s' % inputFile)

            imageData = dict()

            outputFile, _ = os.path.splitext(Path(inputFile).name)
            columnNames = ['LAT', 'LON', 'DAY', 'MEAN_TEMP', 'MAX_TEMP', 'MIN_TEMP', 'DEW_TEMP', 'RAINFALL',
                           'E0', 'ES0', 'ET0', 'RADIATION', 'SNOW_DEPTH', 'CLIMATIC_WATER_BALANCE', 'WINDSPEED', 'VAPOUR_PRESSURE']

            inputYear = re.match('.*(\d{4})\d{4}.*', inputFile).group(1)
            csvData = pd.read_csv(inputFile, header=None, names=columnNames, delimiter=',', index_col=False)
           # csvData = csvData.sort_values(['LON', 'LAT'], ascending=[False, True])
            self.logger.debug(csvData.describe())
            self.logger.debug(csvData.head())

            trLonMin = csvData['LON'].min()
            trLatMax = csvData['LAT'].max()
            self.logger.info('Creating tranformation with coordinates %d, %d' % (trLonMin, trLatMax))
            transform = from_origin(trLonMin, trLatMax, 0.25, 0.25)

            for column in columnNames[3:]:
                self.logger.info('Creating raster for column %s' % column)
                data = csvData.pivot(index='LAT', columns='LON', values=column).sort_index(ascending=False).astype(np.dtype(rasterio.float32)).as_matrix()
                outputPath = outputDir + "/" + column + "/" + inputYear + "/" + outputFile + "_" + column + ".tif"
                imageData['ECMWF_' + column] = self.createGeoTiff(outputPath, data, transform)
        except:
            error = sys.exc_info()[0]
            self.logger.error('Exception occurred: %s' % error)
            return 1

        return imageData

    def createGeoTiff(self, outputPath, data, transform):
        self.logger.info('Writing data to output: %s' % outputPath)

        if not os.path.exists(os.path.dirname(outputPath)):
            os.makedirs(os.path.dirname(outputPath))

        with rasterio.open(outputPath, 'w', driver='GTiff', height=data.shape[0], width=data.shape[1], dtype=rasterio.float32,
                           count=1, crs='EPSG:4326', transform=transform, blockxsize=256, blockysize=256, compress='DEFLATE', tiled=True) as dest:
            dest.write(data, 1)
            dest.update_tags(DateTime=str(datetime.strptime(os.path.basename(outputPath).split('_')[0], '%Y%m%d')))
            self.logger.info('Created GeoTiff')

        return outputPath
