import numpy as np
import re
import shutil
import os

import gdal
gdal.UseExceptions()


def get_lta_files(daily_files, parameter_type):
    lta_files = []
    for f in daily_files:
        match_obj = re.search(r'(\d*).*.tif', f, re.M|re.I)
        lta_files.append('wd1962' + match_obj.group(1)[-4:] + parameter_type + '.tiff')

    return lta_files


def get_agg_filename(files, input_dir):
    match_obj = re.search(r'(\d*)_(\D*).tif', input_dir + "/" + files[0], re.M|re.I)
    start_date = match_obj.group(1)

    match_obj = re.search(r'(\d*)_(\D*).tif', input_dir + "/" + files[len(files)-1], re.M|re.I)
    end_date = match_obj.group(1)

    parameter_type = match_obj.group(2)
    is_rainfall = parameter_type == 'RAINFALL'

    agg_filename = start_date + '_' + end_date + '_' + parameter_type

    return agg_filename, is_rainfall


def copy_file(src_uri, dst_uri):
    f = open(src_uri)
    if not f.closed:
        f.close()
    shutil.copyfile(src_uri, dst_uri)


def init_agg(agg_uri):
    agg = gdal.Open(agg_uri)
    class_rast_array = np.array(agg.GetRasterBand(1).ReadAsArray())
    return agg, class_rast_array


def create_geotiff_from_rast_array(original_ds, rast_array, filename, datatype=gdal.GDT_UInt16):
    band = original_ds.GetRasterBand(1)
    arr = band.ReadAsArray()
    [cols, rows] = arr.shape
    driver = gdal.GetDriverByName("GTiff")
    outdata = driver.Create(filename, rows, cols, 1, datatype)
    outdata.SetGeoTransform(original_ds.GetGeoTransform())
    outdata.SetProjection(original_ds.GetProjection())
    outdata.GetRasterBand(1).WriteArray(rast_array)
    outdata.GetRasterBand(1).SetNoDataValue(-9999)
    outdata.FlushCache()
    outdata = None
    band=None


def aggregate_meteo_files(files, daily_input_dir, weekly_output_dir, lta_input_dir):
    dss = []
    lta_dss = []

    if len(files) == 0:
        return

    agg_filename, is_rainfall = get_agg_filename(files, daily_input_dir)
    agg, class_rast_array = init_agg(os.path.join(daily_input_dir + '/' + files[0][:4], files[0]))

    # load daily raster files for current week
    rast_arrays = []
    for f in files:
        ds = gdal.Open(os.path.join(daily_input_dir + '/' + f[:4], f))
        dss.append(ds)

        rast_array = np.array(ds.GetRasterBand(1).ReadAsArray())
        rast_arrays.append(rast_array)

    if len(dss) == 0:
        return

    # aggregate current week data
    week_agg = np.sum(rast_arrays, axis=0) if is_rainfall else np.mean(rast_arrays, axis=0)

    # get needed LTA filenames based on current week
    lta_files = get_lta_files(files, 'r' if is_rainfall else 't')
    lta_rast_arrays = []
    for lta_file in lta_files:
        ds = gdal.Open(os.path.join(lta_input_dir + '/' + ('R' if is_rainfall else 'T'), lta_file))
        lta_dss.append(ds)

        rast_array = np.array(ds.GetRasterBand(1).ReadAsArray())
        lta_rast_arrays.append(rast_array)

    if len(lta_dss) == 0:
        return

    # aggregate lta data
    lta_week_agg = np.sum(lta_rast_arrays, axis=0) if is_rainfall else np.mean(lta_rast_arrays, axis=0)

    lta_offset_x = 1
    lta_offset_y = -20

    for row in range(20, len(week_agg)):
        for col in range(0, len(week_agg[row])):
            val = week_agg[row][col]
            lta_val = lta_week_agg[row + lta_offset_y][col + lta_offset_x]

            if lta_val > -30000:
                lta_val = lta_val / 100 if is_rainfall else round(lta_val / 10, 2)
                if val < 0.7 * lta_val:
                    class_rast_array[row][col] = 0
                elif val < 0.9 * lta_val:
                    class_rast_array[row][col] = 1
                elif val < 1.1 * lta_val:
                    class_rast_array[row][col] = 2
                elif val < 1.3 * lta_val:
                    class_rast_array[row][col] = 3
                else:
                    class_rast_array[row][col] = 4
            else:
                class_rast_array[row][col] = -9999

    # generate GeoTiff files (weekly data and classification) from raster arrays
    agg_uri_class = os.path.join(weekly_output_dir + "/class", agg_filename + ".tiff")
    agg_uri_data = os.path.join(weekly_output_dir + "/data", agg_filename + ".tiff")
    create_geotiff_from_rast_array(agg, class_rast_array, agg_uri_class)
    create_geotiff_from_rast_array(agg, week_agg, agg_uri_data, gdal.GDT_Float32)

    agg = None
